# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class TrailingSpacesTestCase(RuleTestCase):
    rule_id = 'trailing-spaces'

    def test_disabled(self):
        conf = 'trailing-spaces: disable'
        self.check('', conf)
        self.check('\n', conf)
        self.check('    \n', conf)
        self.check('---\n'
                   'some: text \n', conf)

    def test_enabled(self):
        conf = 'trailing-spaces: {}'
        self.check('', conf)
        self.check('\n', conf)
        self.check('    \n', conf, problem=(1, 1))
        self.check('\t\t\t\n', conf, problem=(1, 1, 'syntax'))
        self.check('---\n'
                   'some: text \n', conf, problem=(2, 11))
        self.check('---\n'
                   'some: text\t\n', conf, problem=(2, 11, 'syntax'))

    def test_with_dos_new_lines(self):
        conf = ('trailing-spaces: {}\n'
                'new-lines: {type: dos}\n')
        self.check('---\r\n'
                   'some: text\r\n', conf)
        self.check('---\r\n'
                   'some: text \r\n', conf, problem=(2, 11))
